@@ -11,6 +11,7 @@ import ssl
 import sys
 import time
 import urllib.parse
+import warnings
 from ssl import CertificateError
 
 import ansicat
@@ -35,7 +36,10 @@ try:
 except (ModuleNotFoundError, ImportError):
     _HAS_CRYPTOGRAPHY = False
 try:
-    import requests
+    with warnings.catch_warnings():
+        # Disable annoying warning shown to LibreSSL users
+        warnings.simplefilter("ignore")
+        import requests
 
     _DO_HTTP = True
 except (ModuleNotFoundError, ImportError):
@@ -237,9 +241,14 @@ def get_cache_path(url, add_index=True):
         # print("Error: %s is not a supported url"%url)
         return None
     if len(cache_path) > 259:
-        print("Path is too long. This is an OS limitation.\n\n")
-        print(url)
-        return None
+        #print("Path is %s characters long which is too long. \
+        #        OS only allows 260 characters.\n\n"%(len(cache_path)))
+        #print(url)
+        #return None
+        # path lenght is limited to 260 charaters. Let’s cut it and
+        # hope that there’s no major conflict here. (that’s still better
+        # than crashing, after all.
+        cache_path = cache_path[:259]
     return cache_path
 
 
@@ -308,6 +317,7 @@ def _fetch_http(
     max_size=None,
     timeout=DEFAULT_TIMEOUT,
     accept_bad_ssl_certificates=False,
+    force_large_download=False,
     **kwargs,
 ):
     if not _DO_HTTP:
@@ -341,10 +351,10 @@ def _fetch_http(
             length = int(response.headers["content-length"])
         else:
             length = 0
-        if max_size and length > max_size:
+        if not force_large_download and max_size and length > max_size:
             response.close()
             return too_large_error(url, str(length / 100), max_size)
-        elif max_size and length == 0:
+        elif not force_large_download and max_size and length == 0:
             body = b""
             downloaded = 0
             for r in response.iter_content():
