@@ -4,7 +4,7 @@
 Offline-First Gemini/Web/Gopher/RSS reader and browser
 """
 
-__version__ = "2.6"
+__version__ = "2.7.1"
 
 # Initial imports and conditional imports {{{
 import argparse
@@ -179,12 +179,10 @@ class GeminiClient(cmd.Cmd):
             "history_size": 200,
             "max_size_download": 10,
             "editor": None,
-            #FIXME: download_images_first is not used anywhere in the code
-            "download_images_first": True,
             "images_mode": "readable",
             "redirects": True,
             # the wikipedia entry needs two %s, one for lang, other for search
-            "wikipedia": "gemini://vault.transjovian.org:1965/search/%s/%s",
+            "wikipedia": "gemini://gemi.dev/cgi-bin/wp.cgi/view/%s?%s",
             "search": "gemini://kennedy.gemi.dev/search?%s",
             "accept_bad_ssl_certificates": False,
             "default_protocol": "gemini",
@@ -298,6 +296,7 @@ class GeminiClient(cmd.Cmd):
         name=None,
         mode=None,
         limit_size=False,
+        force_large_download=False,
     ):
         """This method might be considered "the heart of Offpunk".
         Everything involved in fetching a gemini resource happens here:
@@ -338,8 +337,12 @@ class GeminiClient(cmd.Cmd):
                 match = netloc.endswith(key[1:])
             if match:
                 if self.redirects[key] == "blocked":
-                    text = "This website has been blocked.\n"
-                    text += "Use the redirect command to unblock it."
+                    text = ""
+                    text += "Blocked URL: "+url + "\n"
+                    text += "This website has been blocked with the following rule:\n"
+                    text += key + "\n"
+                    text += "Use the following redirect command to unblock it:\n"
+                    text += "redirect %s NONE" %key
                     if handle and not self.sync_only:
                         print(text)
                     return
@@ -367,6 +370,7 @@ class GeminiClient(cmd.Cmd):
         elif not self.offline_only:
             # A cache is always valid at least 60seconds
             params["validity"] = 60
+        params["force_large_download"] = force_large_download
         # Use cache or mark as to_fetch if resource is not cached
         if handle and not self.sync_only:
             displayed, url = self.opencache.opnk(
@@ -1008,7 +1012,9 @@ class GeminiClient(cmd.Cmd):
 
         output = "Offpunk " + __version__ + "\n"
         output += "===========\n"
-        output += "Highly recommended:\n"
+        output += "System: " + sys.platform + "\n"
+        output += "Python: " + sys.version + "\n"
+        output += "\nHighly recommended:\n"
         output += " - python-cryptography : " + has(netcache._HAS_CRYPTOGRAPHY)
         output += " - xdg-open            : " + has(opnk._HAS_XDGOPEN)
         output += "\nWeb browsing:\n"
@@ -1084,6 +1090,14 @@ class GeminiClient(cmd.Cmd):
         url = self.options["wikipedia"] % (lang, search)
         self._go_to_url(url)
 
+    def do_xkcd(self,line):
+        """Open the specified XKCD comics (a number is required as parameter)"""
+        words = line.split(" ")
+        if len(words) > 0 and words[0].isalnum():
+            self._go_to_url("https://xkcd.com/%s"%words[0])
+        else:
+            print("Please enter the number of the XKCD comic you want to see")
+
     def do_gus(self, line):
         """Submit a search query to the geminispace.info search engine."""
         if not line:
@@ -1210,7 +1224,6 @@ Use "view XX" where XX is a number to view information about link XX.
             else:
                 self.opencache.opnk(u, terminal=False)
 
-    @needs_gi
     def do_shell(self, line):
         """Send the content of the current page to the shell and pipe it.
         You are supposed to write what will come after the pipe. For example,
@@ -1222,10 +1235,15 @@ Use "view XX" where XX is a number to view information about link XX.
         """
         # input is used if we wand to send something else than current page
         # to the shell
-        tmp = self.opencache.get_temp_filename(self.current_url)
+        tmp = None
+        if self.current_url:
+            tmp = self.opencache.get_temp_filename(self.current_url)
         if tmp:
             input = open(tmp, "rb")
             run(line, input=input, direct_output=True)
+        else:
+            run(line,direct_output=True)
+
 
     @needs_gi
     def do_save(self, line):
@@ -1919,7 +1937,8 @@ Use "view XX" where XX is a number to view information about link XX.
                 return False
 
         def fetch_url(
-            url, depth=0, validity=0, savetotour=False, count=[0, 0], strin=""
+            url, depth=0, validity=0, savetotour=False, count=[0, 0], strin="",
+            force_large_download=False
         ):
             # savetotour = True will save to tour newly cached content
             # else, do not save to tour
@@ -1940,7 +1959,8 @@ Use "view XX" where XX is a number to view information about link XX.
                 print(toprint, end=endline)
                 # If not saving to tour, then we should limit download size
                 limit = not savetotour
-                self._go_to_url(url, update_hist=False, limit_size=limit)
+                self._go_to_url(url, update_hist=False, limit_size=limit,\
+                        force_large_download=force_large_download)
                 if savetotour and isnew and netcache.is_cache_valid(url):
                     # we add to the next tour only if we managed to cache
                     # the ressource
@@ -1979,7 +1999,8 @@ Use "view XX" where XX is a number to view information about link XX.
                         )
 
         def fetch_list(
-            list, validity=0, depth=1, tourandremove=False, tourchildren=False
+            list, validity=0, depth=1, tourandremove=False, tourchildren=False,
+            force_large_download=False
         ):
             links = self.list_get_links(list)
             end = len(links)
@@ -1994,6 +2015,7 @@ Use "view XX" where XX is a number to view information about link XX.
                     validity=validity,
                     savetotour=tourchildren,
                     count=[counter, end],
+                    force_large_download=force_large_download
                 )
                 if tourandremove:
                     if add_to_tour(l):
@@ -2021,13 +2043,14 @@ Use "view XX" where XX is a number to view information about link XX.
         starttime = int(time.time())
         for l in subscriptions:
             fetch_list(l, validity=refresh_time, depth=depth, tourchildren=True)
-        # Then the fetch list (item are removed from the list after fetch)
+        # Then the to_fetch list (item are removed from the list after fetch)
         # We fetch regarless of the refresh_time
         if "to_fetch" in lists:
             nowtime = int(time.time())
             short_valid = nowtime - starttime
             fetch_list(
-                "to_fetch", validity=short_valid, depth=depth, tourandremove=True
+                "to_fetch", validity=short_valid, depth=depth, tourandremove=True,
+                force_large_download=True
             )
         # then we fetch all the rest (including bookmarks and tour)
         for l in normal_lists:
